<?php
// $Id$

function _nfa_securitycheck($isUninstall=FALSE, $onlyCheck=FALSE) {
  global $base_root, $base_path;
  
  // Check the folder permissions:
  //        Set to 0701 whatever is there in "nfa_folders"
  //        Change back the permissions of folders that have been removed from vigilance
  // Check for redirect statements in .htaccess file
  
  if ($isUninstall)
    $nfa_folders = array();
  else
    $nfa_folders = variable_get("nfa_folders", array());
  $nfa_perms_to_reset = variable_get("nfa_original_perms", array());
  
  $redirect_statements_1 = "\n<IfModule !mod_rewrite.c>\n <IfModule mod_alias.c>\n";
  $redirect_statements_2 = "\n<IfModule mod_rewrite.c>\n RewriteEngine On\n";
  $nfa_original_perms = array();
  foreach ($nfa_folders as $this_folder) {
    $this_folder['path'] = _nfa_correct_path($this_folder['path']);
    $current_perm = fileperms($this_folder['path']) & 0777;
    if ($current_perm != 0701) {
      if (!$onlyCheck) {
        $nfa_original_perms[$this_folder['path']] = $current_perm;
        $res = chmod($this_folder['path'], 0701);
        //print("<br/>".$this_folder['path']." - 0700<br/>");
        if (!$res)
          drupal_set_message(t('WARNING: Could not change permission of folder ' . $this_folder['path'] . '.'), 'error');
      }
      else
        drupal_set_message(t('Node File Access Security Alert: The permission to folder ' . $this_folder['path'] . ' is not set properly.'), 'warning');
    }
    $nfa_perms_to_reset[$this_folder['path']] = FALSE;
    $this_subs = url("nodefileaccess", array('absolute'=>TRUE,'query'=>"file=" . $this_folder['path']));
    $redirect_statements_1 .= "  Redirect " . $base_path . $this_folder['path'] . " " . $this_subs . "\n";
    $redirect_statements_2 .= " RewriteRule ^\\Q" . $this_folder['path'] . "\\E(.*)\$ " . $this_subs . "\$1\n";
  }
  $redirect_statements_1 .= " </IfModule>\n</IfModule>\n";
  $redirect_statements_2 .= "</IfModule>\n";
  
  foreach ($nfa_perms_to_reset as $path=>$perm)
    if ((fileperms($path) & 0777) != 0701 && $perm !== FALSE && !$onlyCheck) {
      $res = chmod($path, $perm);
      print("<br/>".$path." -  ".$perm."<br/>");
      if (!$res)
        drupal_set_message(t('WARNING: Could not set permission of folder ' . $this_folder['path'] . '.'), 'error');
    }
  
  variable_set("nfa_original_perms", $nfa_original_perms);
  
  $redirect_statements_prefix = "\n## DO NOT CHANGE THE CONTENTS OF THIS BLOCK" .
                                "\n## ========================================" .
                                "\n## NodeFileAccess redirect rules: BEGIN\n";
  $redirect_statements_suffix = "\n## NodeFileAccess redirect rules: END" .
                                "\n## ========================================\n";
  
  if ($isUninstall)
    $redirect_statements_Full = "";
  else
    $redirect_statements_Full = $redirect_statements_prefix . $redirect_statements_2 . $redirect_statements_1 . $redirect_statements_suffix;
  
  $htaccess_content = file_get_contents(".htaccess");
  if (preg_match("/\\Q" . $redirect_statements_prefix . "\\E(.*)\\Q" . $redirect_statements_suffix . "\\E/s", $htaccess_content, $matches)==1)
    $htaccess_content = preg_replace("/\\Q" . $redirect_statements_prefix . "\\E.*\\Q" . $redirect_statements_suffix . "\\E/s",
                      str_replace("\$1","\\\$1",$redirect_statements_Full), $htaccess_content);
  else
    $htaccess_content .= $redirect_statements_Full;
  
  if ($onlyCheck && $matches[0] != $redirect_statements_Full)
    drupal_set_message(t('Node File Access Security Alert: The redirects in .htaccess are not set properly.'), 'warning');
  else
  {
    $fp = fopen(".htaccess", 'wb');
    if($fp === FALSE || fwrite($fp, $htaccess_content) === FALSE)
      drupal_set_message(t('WARNING: Could not wite to .htaccess .'), 'error');
    fclose($fp);
  }
}

// ==============================================

function _nfa_find_association_and_redirect($file = NULL, &$thenode = NULL) {
  global $nfa_hook_file_download_complete;
  $nfa_hook_file_download_complete = TRUE;
  
  if (empty($file))
    $file = $_GET['file'];
  
  $fileQuery = _nfa_correct_path($file);
  $r = db_query("SELECT nid FROM {nfa_association} WHERE BINARY file='%s'", $fileQuery);
  $row = db_fetch_array($r);
  if(empty($row) || !isset($row['nid']) || intval($row['nid']) == 0) {
    if (variable_get("nfa_download_unassociated_files", TRUE))
      file_download($file); // If a file is not associated with a node, just transfer the control to dowload modules
    else
      { drupal_access_denied(); exit(); }
  }
  elseif (intval($row['nid']) < 0 || !user_access("view NFA files")) { drupal_access_denied(); exit(); }
  else
    drupal_goto("node/" . $row['nid'], "file=" . $file);

}

// ==============================================

function _nfa_check_association_and_download($nid) {
  global $nfa_hook_file_download_complete;
  $nfa_hook_file_download_complete = TRUE;
  
  $file = $_GET['file'];
  $fileQuery = _nfa_correct_path($file);
  $r = db_result(db_query("SELECT COUNT(*) FROM {nfa_association} WHERE nid=%d AND BINARY file='%s'", $nid, $fileQuery));
  if($r && !empty($r) && $r>0)
    file_download($file);  // Download the file
  else
    { drupal_access_denied(); exit(); }
}

// ==============================================

function _nfa_force_update_associations_all($forceUpdate = TRUE) {
  $totalNIDcount = db_result(db_query("SELECT COUNT(DISTINCT nid) FROM {node}")) + 2;
  $result = db_query("SELECT DISTINCT nid FROM {node}");
  if ($result == FALSE && db_error())
    drupal_set_message(t('Failed to query node.'), 'warning');

  $count = 0;
  while ($thisrow = db_fetch_array($result)) {
    _nfa_update_associations(intval($thisrow['nid']), $forceUpdate);
    $count++;
    variable_set("nfa_progress", intval(100*$count/$totalNIDcount));
    //sleep(1);
  }
  for ($nnid=-1; $nnid<=0; $nnid++) {
    _nfa_update_associations($nnid, $forceUpdate);
    $count++;
    variable_set("nfa_progress", intval(100*$count/$totalNIDcount));
    //sleep(1);
  }
  
  variable_set("nfa_progress", 100);
}

function _nfa_update_associations($nid, $forceUpdate = FALSE) {
  $nid = intval($nid);
  
  $node_update_timestamp = db_result(db_query("SELECT changed FROM {node} WHERE nid=%d", $nid));
  
  if ($nid <= 0 || $node_update_timestamp === FALSE || $node_update_timestamp === NULL)  // The node is a virtual node or it has been deleted
  {
    // Just check if the associated files exist
    $result_existingfiles = db_query("SELECT file FROM {nfa_association} WHERE nid=%d", $nid);
    while ($res_row = db_fetch_array($result_existingfiles)) {
      $thisfile = _nfa_correct_path($res_row['file']);
      if (empty($thisfile))
        $temp_res = db_query("DELETE FROM {nfa_association} WHERE file='%s' AND nid=%d", $res_row['file'], $nid);
    }
    return;
  }
  
  $ass_update_timestamp = db_result(db_query("SELECT ass_update_timestamp FROM {nfa_asstimestamp} WHERE nid=%d", $nid));
  if (empty($ass_update_timestamp)) {
    $result = db_query("INSERT INTO {nfa_asstimestamp} SET nid=%d, ass_update_timestamp=%d", $nid, 0);
    if ($result==FALSE && db_error())
      drupal_set_message(t('Failed to insert into nfa_asstimestamp.'), 'error');
    $ass_update_timestamp = 0;
  }
  
  if ($ass_update_timestamp <= $node_update_timestamp || $forceUpdate) // If contents of the node has changed
  {
    // Make a list of existing associations for this node
    $result_existingfiles = db_query("SELECT file FROM {nfa_association} WHERE nid=%d", $nid);
    if ($result_existingfiles == FALSE && db_error())
      drupal_set_message(t('Failed to read nfa_association.'), 'error');
    
    // Search for and create new associations
    $allFoundFiles = array();
    $nfa_association_finder = variable_get("nfa_association_finder", array());
    foreach ($nfa_association_finder as $this_ass_finder) {
      switch ($this_ass_finder['type']) {
        case 'filepath':
          $querystr = str_replace("\$nid", "" . $nid, $this_ass_finder['querystring']);
          $result = db_query($querystr);
          while ($thisrow = db_fetch_array($result)) {
            $thisrow['filepath'] = _nfa_correct_path($thisrow['filepath']);
            $allFoundFiles[] = $thisrow['filepath'];
            _nfa_set_association($thisrow['filepath'], $nid);
          }
          break;
        case 'folderpath':
          $querystr = str_replace("\$nid", "" . $nid, $this_ass_finder['querystring']);
          $result = db_query($querystr);
          while ($thisrow = db_fetch_array($result)) {
            $filepathList = _nfa_get_files_from_folder_recursive($thisrow['folderpath']);
            foreach ($filepathList as $thisfilepath) {
              $thisfilepath = _nfa_correct_path($thisfilepath);
              $allFoundFiles[] = $thisfilepath;
              _nfa_set_association($thisfilepath, $nid);
            }
          }
          break;
        case 'html':
          $querystr = str_replace("\$nid", "" . $nid, $this_ass_finder['querystring']);
          $result = db_query($querystr);
          while ($thisrow = db_fetch_array($result)) {
            // $thisrow = db_fetch_array($result);
            $filepathList = _nfa_find_files_from_HTML($thisrow['html']);
            foreach ($filepathList as $thisfilepath) {
              $thisfilepath = _nfa_correct_path($thisfilepath);
              $allFoundFiles[] = $thisfilepath;
              _nfa_set_association($thisfilepath, $nid);
            }
          }
          break;
        case 'PHP-filepaths': // Use with care!
          $filepathList = eval($this_ass_finder['querystring']);
          if (is_array($filepathList))
            foreach ($filepathList as $thisfilepath) {
              $thisfilepath = _nfa_correct_path($thisfilepath);
              $allFoundFiles[] = $thisfilepath;
              _nfa_set_association($thisfilepath, $nid);
            }
          break;
        case 'PHP-folderpaths': // Use with care!
          $folderpathList = eval($this_ass_finder['querystring']);
          if (is_array($folderpathList))
            foreach ($folderpathList as $thisfolderpath) {
              $filepathList = _nfa_get_files_from_folder_recursive($thisfolderpath);
              foreach ($filepathList as $thisfilepath) {
                $thisfilepath = _nfa_correct_path($thisfilepath);
                $allFoundFiles[] = $thisfilepath;
                _nfa_set_association($thisfilepath, $nid);
              }
            }
          break;
        case 'PHP-htmls': // Use with care!
          $htmlList = eval($this_ass_finder['querystring']);
          if (is_array($htmlList))
            foreach ($htmlList as $thisHTML) {
              $filepathList = _nfa_find_files_from_HTML($thisHTML);
              foreach ($filepathList as $thisfilepath) {
                $thisfilepath = _nfa_correct_path($thisfilepath);
                $allFoundFiles[] = $thisfilepath;
                _nfa_set_association($thisfilepath, $nid);
              }
            }
          break;
      }
    }
    
    // Remove the associations that are no more in use at this node
    $ass_to_remove = array();
    if (variable_get("nfa_auto_remove_unlocked_associations", TRUE))
      $ass_to_remove = array('-1','0');
    if (variable_get("nfa_auto_remove_locked_associations", FALSE))
      $ass_to_remove[] = '1';
    if (!empty($ass_to_remove)) {
      $ass_list_to_remove = implode(",", $ass_to_remove);
      while ($res_row = db_fetch_array($result_existingfiles))
        if (!in_array($res_row['file'], $allFoundFiles)) {
          $result_remove = db_query("DELETE FROM {nfa_association} WHERE nid=%d AND BINARY file='%s' AND locked IN (" . $ass_list_to_remove . ")", $nid, $res_row['file']);
          if ($result_remove == FALSE && db_error())
            drupal_set_message(t('Could not clean nfa_association.'), 'warning');
        }
    }
    
    // Update timestamp
    $result = db_query("UPDATE {nfa_asstimestamp} SET ass_update_timestamp=%d WHERE nid=%d", time(), $nid);
    if ($result == FALSE && db_error())
      drupal_set_message(t('Failed to update nfa_asstimestamp.'), 'error');
  }
}

// -----------------

function _nfa_get_files_from_folder_recursive($fpath) {
  $fileList = array();
  $fpath = _nfa_correct_path($fpath);
  /*
  $handle = opendir($fpath);
  if ($handle)
    while (FALSE !== ($thisfile = readdir($handle))) {
      if ($thisfile == "." || $thisfile == "..")
        continue;
      $thisfile = $fpath.$thisfile;
      if (is_file($thisfile))
        $fileList[] = $thisfile;
      elseif (is_dir($thisfile))
        $fileList = array_merge($fileList, _nfa_get_files_from_folder_recursive($thisfile));
    } */
  $fileListObj = file_scan_directory($fpath, ".*");
  $fileList = array_keys($fileListObj);
    
  return $fileList;
}

// -----------------

function _nfa_correct_path($rawpath) {
  global $base_path, $base_root, $base_url;
  
  $rawpath = trim(ltrim($rawpath,"/\\"));
  // Check if base_path is appended at the beginning
  $my_base_path = ltrim($base_path,"/\\");
  if (!(is_dir($rawpath) || is_file($rawpath)) && substr($rawpath,0,strlen($my_base_path)) == $my_base_path )
    $rawpath = substr($rawpath, strlen($my_base_path));
  // URL decode?
  if (!(is_dir($rawpath) || is_file($rawpath)))
    $rawpath = urldecode($rawpath);
  // Standardize the path
  $rawpath = realpath($rawpath);
  $currentpath = realpath(".");
  if (substr($rawpath,0,strlen($currentpath)) == $currentpath)
    $rawpath = substr($rawpath,strlen($currentpath));
  $rawpath = ltrim($rawpath,"/\\");
  // Place a / at the end of a folder path
  if (is_dir($rawpath) && substr($rawpath,-1) !== "/")
    $rawpath = $rawpath."/";
  
  return $rawpath;
}

// -----------------

function _nfa_set_association($filepath, $nid, $lock = 1, $checkassid = NULL, $forceChange = FALSE) {
  $orgFilepath = $filepath;
  $filepath = _nfa_correct_path($filepath);
  
  $res = db_query("SELECT locked, nid, assid FROM {nfa_association} WHERE BINARY file='%s'", $filepath);
  if ($thisRow = db_fetch_array($res))
    $is_locked = intval($thisRow['locked']); // -1, 0 or 1
  else
    $is_locked = FALSE;
  
  if (empty($filepath)) // The file does not exist!!
  {
    if ($is_locked!==FALSE)
      $result = db_query("DELETE FROM {nfa_association} WHERE BINARY file='%s'", $filepath);
    drupal_set_message(t('The path ' . $orgFilepath . ' is not a valid file.'), 'warning');
    return(FALSE);
  }
  
  if ($checkassid !== NULL && $is_locked !== FALSE && $checkassid != intval($thisRow['assid'])) {
    drupal_set_message(t('Association ID mismatch.'), 'warning');
    return(FALSE);
  }
  
  if ($is_locked === FALSE) // The file does not exist in the DB
  {
    $r = db_query("INSERT INTO {nfa_association} SET file='%s', nid=%d, locked=%d", $filepath, $nid, $lock);
    if ($r == FALSE && db_error())
      drupal_set_message(t('Failed to insert into nfa_association.'), 'error');
  }
  elseif ($forceChange || $is_locked === 0 || ($is_locked === -1 && $nid !== intval($thisRow['nid'])) ) {
    $r = db_query("UPDATE {nfa_association} SET nid=%d, locked=%d WHERE BINARY file='%s'", $nid, $lock, $filepath);
    if ($r == FALSE && db_error())
      drupal_set_message(t('Failed to update nfa_association.'), 'error');
    // drupal_set_message(t('Updated '.$filepath.': '.$forceChange.', '.$is_locked.', '.$nid.', '.$thisRow['nid'].', '.$lock.' .'), 'error');
  }
}

// -----------------

function _nfa_find_files_from_HTML($theHTML) {
  global $base_path, $base_root, $base_url;
  
  $filepaths = array();
  $nfa_folders = variable_get("nfa_folders", array());
  foreach ($nfa_folders as $thisfolder) {
    $this_folder = $thisfolder['path'];
    $rules = explode("\n", str_replace("\r", "\n", $thisfolder['pattern']));
    foreach ($rules as $thisrule) {
      if (empty($thisrule))
        continue;
      $thisrule = str_replace(array("\$base_path","\$base_root","\$base_url","\$this_folder"),
                  array($base_path, $base_root, $base_url, $this_folder), $thisrule);
      $ruleParts = explode("=>", $thisrule);
      if (count($ruleParts) != 2) {
        drupal_set_message(t('Incorrect syntax in a Pattern Rule for folder ' . $this_folder . '.'), 'error');
        continue;
      }
      $regExp = trim($ruleParts[0]);
      $subsPat = trim($ruleParts[1]);
      $res = preg_match_all($regExp, $theHTML, $matches, PREG_SET_ORDER);
      if (!is_array($matches)) {
        drupal_set_message(t('Invalid regular expression: ' . $regExp . ' .'), 'error');
        continue;
      }
      foreach ($matches as $thismatch) {
        $thisfilepath = $subsPat;
        foreach ($thismatch as $k=>$m)
          $thisfilepath = str_replace("\$" . $k, $m, $thisfilepath);
        $filepaths[] = $thisfilepath;
      }
    }
  }
  
  return $filepaths;
}

// ======================================
// Implementation of a very simple progress bar

function _nfa_add_js_progressbar (&$form) {
  
  $form['progress_iframe'] = array(
        '#value' => "<div id='progress_div' style='border: 1px solid #888888; margin:5px; padding: 5px; font-weight: bold; font-size: 12pt;display:none;'>Building associations. Please wait.
              <iframe id='progress_iframe' src='" . url('get_progress', array('query' => 'onlyval=0')) . "' style='width:350px; height:20px; margin-left:10px; overflow:hidden;' marginheight='0' marginwidth='0' frameborder='0' scrolling='no'>
                <div style='border: 1px solid #888888; background-color: #aaaaaa; margin:5px; padding: 0px; width:350px; height:20px; position:relative;'><div id='nfa_progress_bar_progress' style='border: 0px; background-color: #44ee22; margin:0px; padding: 0px; width:3px; height:20px; position:absolute; left:0px; top:0px;'></div></div>
              </iframe></div>"
      );

  $ProgressBarJS = "
            function SubmitAndShowProgress(theElem) {
              
              var branchLevel = 0;
              var myElems = new Array();
              myElems[0] = theElem;
              while (myElems[branchLevel].tagName.toUpperCase() != 'FORM') {
                branchLevel++;
                myElems[branchLevel] = myElems[branchLevel-1].parentNode;
              }
              theform = myElems[branchLevel];
              
              document.getElementById('progress_div').style.display = 'block';
              var inpElms = theform.getElementsByTagName('input');
              for (i=0; i<inpElms.length; i++)
                if (inpElms[i].getAttribute('type').toUpperCase() == 'SUBMIT')
                  inpElms[i].style.display = 'none';
              
              // AJAX progress display
              setTimeout('recursiveUpdateWidth()', 0);
              
              // Submit the form
              if (branchLevel==0) {
                theform.submit();
                return false;
              }
              else
                return true;
            }
            
            
            function recursiveUpdateWidth()  // This is used only if the iframe fails
            {
              if (window.XMLHttpRequest)
                var oRequest = new XMLHttpRequest();
              else
                var oRequest = new ActiveXObject('Microsoft.XMLHTTP');

              oRequest.open('GET','index.php?q=get_progress&onlyval=1',true);
              
              oRequest.onreadystatechange = function() { 
                if(oRequest.readyState == 4) {
                  var progress = parseInt(oRequest.responseText);
                  document.getElementById('nfa_progress_bar_progress').style.width = parseInt(progress*3.5) + 'px';
                  if (isNaN(progress) || progress < 100)
                    setTimeout('recursiveUpdateWidth()', 250);
                }
              };
              oRequest.send(null);
            } ";
            
  drupal_add_js($ProgressBarJS, 'inline');

}


function _nfa_get_progress() {
  $theProgress = intval(variable_get("nfa_progress", 100));
  if ($theProgress == 100)
    variable_set("nfa_progress", 0);
  $onlyVal = intval($_GET['onlyval']);
  
  if ($onlyVal == 1) {
    drupal_set_header('Content-type: text/plain');
    print($theProgress);
    exit();
  }
  else {
    print("<html><body style='margin:0;padding:0;background-color:#888888;' onload='javascript:recursiveUpdateWidth()'>");
    print("<div id='nfa_progress_bar_progress' style='border:0px;background-color:#44ee22;margin:0px;padding:0px;width:3px;height:20px; position:absolute;left:0px;top:0px;'></div>");
    print("\n<SCRIPT language='JavaScript'>
            function recursiveUpdateWidth() {
              if (window.XMLHttpRequest)
                var oRequest = new XMLHttpRequest();
              else
                var oRequest = new ActiveXObject('Microsoft.XMLHTTP');

              oRequest.open('GET','index.php?q=get_progress&onlyval=1',true);
              
              oRequest.onreadystatechange = function() { 
                if(oRequest.readyState == 4) {
                  var progress = parseInt(oRequest.responseText);
                  document.getElementById('nfa_progress_bar_progress').style.width = parseInt(progress*3.5) + 'px';
                  if (isNaN(progress) || progress < 100)
                    setTimeout('recursiveUpdateWidth()', 250);
                }
              };
              oRequest.send(null);
            }
        </SCRIPT>\n");
    print("</body></html>");
    exit();
  }
}

